#!/usr/local/bin/guile -s
!#
;;;; Web Gallery Display Engine
;;;; Copyright (C) 2004 Sergey Poznyakoff
;;;; 
;;;; This program is free software; you can redistribute it and/or modify
;;;; it under the terms of the GNU General Public License as published by
;;;; the Free Software Foundation; either version 2 of the License, or
;;;; (at your option) any later version.
;;;;
;;;; This program is distributed in the hope that it will be useful,
;;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;;; GNU General Public License for more details.
;;;;
;;;; You should have received a copy of the GNU General Public License
;;;; along with this program; if not, write to the Free Software
;;;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
;;;;

(use-modules (www cgi))
(use-modules (ice-9 regex))
(cgi:init)

(define cgi-url "cgi-bin/webgal.cgi")
(define gallery-template-file "gallery.html")
(define image-template-file "image.html")
(define index-template-file "idx.html")
(define gallery-index-url-prefix "preview")
(define gallery-index-dir "preview") 
(define gallery-base "gallery")
(define gallery-base-dir "gallery")
(define image-rx "(.*)\.(jpg)")
(define text-rx "(.*)\.(txt)")
(define max-columns 6)

(define (gallery-id) 
  (cgi:value "gallery"))

(define (image-id)
  (cgi:value "image"))
  
(define (merge-img-txt img-list txt-list)
  (let ((result '()))
    (for-each
     (lambda (x)
       (set! result
	     (cons
	      (cons
	       (string-append (car x) "." (cdr x))
	       (cond
		((assoc (car x) txt-list) =>
		 (lambda (name)
		   (string-append (car name) "." (cdr name))))
		(else
		 #f)))
	      result)))
     img-list)
    (reverse result)))

(define (make-index-dir-name)
  (let ((gallery (gallery-id)))
    (if gallery
	(string-append gallery-base-dir "/" gallery "/" gallery-index-dir)
	(throw 'no-gallery-id))))

(define (scan-index-dir dir-name)
;  (display "SCANNING ")(display dir-name)(display "<br>")
  (letrec ((d (opendir dir-name))
	   (car<? (lambda (a b)
		    (string<? (car a) (car b)))))
    (let ((result (do ((dir (readdir d) (readdir d))
		       (img-list '())
		       (txt-list '()))
		      ((eof-object? dir)
		       (merge-img-txt (stable-sort img-list car<?)
				      (stable-sort txt-list car<?)))
		    (cond
		     ((string-match image-rx dir) =>
		      (lambda (match)
			(set! img-list (cons (cons
					      (match:substring match 1)
					      (match:substring match 2))
					     img-list))))
		     ((string-match text-rx dir) =>
		      (lambda (match)
			(set! txt-list (cons (cons
					      (match:substring match 1)
					      (match:substring match 2))
					     txt-list))))))))
      (closedir d)
;      (display "RESULT ")(display result)(display "<br>")
      result)))

(define (read-short name . rest)
  (let ((print? (or (null? rest) (car rest)))
	(found? #f))
    (call-with-current-continuation
     (lambda (return)
       (do ((line (read-line) (read-line)))
	   ((eof-object? line) (cons found? line))
	 (cond
	  ((string-match "short: *" line) =>
	   (lambda (match)
	     (cond
	      (print?
	       (set! found? #t)
	       (display " ")
	       (display (match:suffix match))))))
	  (else
	   (return (cons found? line)))))))))

(define (read-file name . rest)
  (with-input-from-file name
    (lambda ()
      (letrec ((display-rest (lambda (x)
			       (do ((line x (read-line)))
				   ((eof-object? line) #f)
				 (display " ")
				 (display line)))))
	
	(if (and (not (null? rest)) (car rest))
	    (let ((res (read-short name)))
	      (if (not (car res))
		  (display-rest (cdr res))))
	    (let ((res (read-short name #f)))
	      (display-rest (cdr res))))))))

(define (find-image-neighbors name)
  (call-with-current-continuation
   (lambda (return)
     (do ((flist (scan-index-dir (make-index-dir-name)) (cdr flist))
	  (prev #f))
	 ((null? flist) (cons prev #f))
       (let ((item (caar flist)))
	 (if (string=? item name)
	     (return (cons prev (and (not (null? (cdr flist)))
				     (cadr flist)))) 
	     (set! prev (car flist))))))))

(define image-neighbors #f)

(define (current-image-neighbors)
  (if (not image-neighbors)
      (set! image-neighbors (find-image-neighbors (image-id))))
  image-neighbors)

(define (main-form)
  (display "<FORM ACTION=\"")
  (display cgi-url)
  (display "\" METHOD=POST>")
  
  (let ((filelist (scan-index-dir (make-index-dir-name))))
    (display "<div align=\"center\" class=\"gallery\"><table class=\"gallery\" rules=\"groups\">")
    (do ((row 0 (1+ row)))
	((null? filelist) #f)
      (display "<tbody>\n")
      (display "<tr class=\"image\">")
      
      (do ((col 0 (1+ col))
	   (flist filelist (cdr flist)))
	  ((or (null? flist) (= col max-columns)) #f)
	(display "<td class=\"image\" align=\"center\"><a href=\"")

        (display (image-url (gallery-id) (car flist)))

	(display "\"><img border=\"0\" src=\"")
	(display (string-append
		  gallery-base "/"
		  (gallery-id) "/"
		  gallery-index-url-prefix "/" (caar flist)))
	(display "\"></img></a></td>"))
      
      (display "</tr>\n<tr class=\"comment\">")

      (do ((col 0 (1+ col)))
	  ((or (null? filelist) (= col max-columns)) #f)
	(display "<td class=\"comment\">")
	(if (cdar filelist)
	    (read-file (string-append
			gallery-base-dir
			"/"
			(gallery-id)
			"/"
			gallery-index-dir
			"/" (cdar filelist))
		       #t))
	(display "</td>")
	(set! filelist (cdr filelist)))

      (display "</tr>")
      (newline)
      (display "</tbody>")
      (newline))
    (display "</table></div>")))

(define (image-form image)
  (display "<div align=\"center\">")
  (display "<img src=\"")
  (display (string-append gallery-base "/" (gallery-id) "/" image))
  (display "\"></img></div>"))

(define (image-url id img)
  (apply string-append 
          (append
           (list
            cgi-url
            "?gallery=" id
            "&amp;"
            "image=" (car img))
	   (cond 
	    ((cdr img) => (lambda (t)
	                    (list
                               "&amp;"
		               "text=" t)))
            (else 
             '())))))

(define explist
  (list (cons "@@program-name@@"  (lambda (id)
				    (display cgi-url)))
	(cons "@@gallery-url@@"   (lambda (id)
				    (display cgi-url)
				    (display "?gallery=")
				    (display id)))
	(cons "@@gallery-descr@@"  (lambda (id)
				     (let ((name (string-append
						  gallery-base-dir "/" id "/" 
						  gallery-index-dir "/" "TITLE")))
				       (if (access? name R_OK)
					   (read-file name)))))))
			    
(define (expand-template template dir)
  (let loop ((template template))
    (cond
     ((string-index template #\@) =>
      (lambda (w)
	(display (substring template 0 w))
	(if (and (< (+ w 2) (string-length template))
		 (char=? (string-ref template (1+ w)) #\@))
	    (let ((end-pos (string-index template #\@ (+ w 2))))
	      (if (and end-pos
		       (< (1+ end-pos) (string-length template))
		       (char=? (string-ref template (1+ end-pos)) #\@))
		  (let* ((name (substring template w (+ end-pos 2)))
			 (entry (assoc name explist)))
		    (cond
		     (entry
		      ((cdr entry) dir)
		      (loop (substring template (+ end-pos 2))))
		     (else
		      (display "@@")
		      (loop (substring template (+ w 2))))))
		  (begin
		    (display "@")
		    (loop (substring template (+ w 1))))))
	    (begin
	      (display "@")
	      (loop (substring template (1+ w)))))))
     (else
      (display template)))))
	     
	
(define (make-gallery-index template)
  (let ((dir (opendir gallery-base-dir)))
    (do ((d (readdir dir) (readdir dir)))
	((eof-object? d) #f)
      (if (not (or (string=? d ".") (string=? d "..")))
	  (catch #t
		 (lambda ()
		   (let ((st (stat (string-append gallery-base-dir "/" d))))
		     (case (stat:type st)
		       ((directory)
			(expand-template template d)))))
		 (lambda args
		   #f))))))
		     
(define (make-html)
  (let ((accumulator #f)
        (mode #f))
    (do ((line (read-line) (read-line)))
	((eof-object? line) #f)
      (cond
       (accumulator
	(cond
	 ((string=? line "@@end@@")
	  (make-gallery-index accumulator)
          (set! accumulator #f))
         ((and (>= (string-length line) 2)
               (string=? (substring line 0 2) "?>")) 
          (eval-string accumulator)
	  (set! accumulator #f))
	 (else
	  (set! accumulator (string-append accumulator "\n" line)))))
       ((string=? line "@@title@@")
	(let ((name (string-append
		     gallery-base-dir "/" (gallery-id) "/" 
		     gallery-index-dir "/" "TITLE")))
	  (if (access? name R_OK)
	      (read-file name))))
       ((string=? line "@@gallery@@")
	(main-form))
       ((and (>= (string-length line) 7)
             (string=? (substring line 0 7) "<?guile"))
        (set! accumulator (substring line 7)))
       ((string=? line "@@image@@")
	(image-form (image-id)))
       ((string=? line "@@text@@")
	(cond
	 ((cgi:value "text") =>
	  (lambda (x)
	    (read-file (string-append
			gallery-base-dir "/" (gallery-id) "/" 
			gallery-index-dir "/" x))))))
       ((string=? line "@@repeat@@")
	(set! accumulator ""))
       (else
	(expand-template line (gallery-id)) 
	(newline))))))  

;;; Main
(display "Content-type: text/html; charset=utf-8\r\n\r\n")

(catch #t
       (lambda ()
	 (let ((image (image-id))
	       (gallery (gallery-id)))
	   (with-input-from-file
	       (cond
		(image image-template-file)
		(gallery gallery-template-file)
		(else index-template-file))
	     make-html))) 
       (lambda args
	 (display "Exception caught: ")
	 (display args)))


;;;; Local variables:
;;;; mode: Scheme
;;;; buffer-file-coding-system: utf-8
;;;; End:
